# Description:  This is a pipeline that generates PAR-CLIP reads.
#				This is achieved by simulating RNA-Seq reads and
#				mimicking RBP binding. Therefore, the coordinates
#				of RBP motifs are learned from the transcriptome.
#				Subsequently, reads from the RNA-Seq simulation
#				overlapping a motif are selected and finally mutated.

# Contact: wanja.kassuhn@web.de

# todo: 

# req:  fimo
#       flux    

# input:    - the genome in separated fasta files
#			- an gtf file containing the an annotation
#			- a transcriptiome in fasta format following a specific header format 
#			- a fimo input file following their structure (alphabet, background distribution, PWM of motif) 
#			- a file with position specific mutation rates for conversions, deletions and/or truncations depending on introduces diagnostic events



#!/bin/bash

args=("$@")

set -e

#INIT
	
genomeDir=	   		# PATH to genome file in fasta and *.genome file with same name (Optional as gziped file: genome.tar.gz)
outDir=		   		# PATH to output
gtf=           		# Name of the annotation file
motifLen=      		# Length of the motif
inputDir=      		# PATH to input directory containing all input files (annotation, transcriptome, flux parameter template, pattern and mutation rate files)
motiffile=     		# Name of the FIMO input file
transcriptID=       # Transcript ID of transcript to simulate reads from
motifIndex=         # FIMO output file
expProf=            # Expression profile

#DEFAULT SETTINGS

mode='simulateReads' # generateFiles or simulateReads
ends='NO'      		# YES or NO for paired end reads
motif='specific'    # Name of the RBP
readLength=30  		# Read length
fasta='YES'    		# Create FASTA file
numR=2000000000  	# Number of reads to be created
libname='LIB'  		# Name of the library file
parname='Runx'      # Name of the run that is included in the output files
useMemory=16        # Available memory
mutMode='C'			# Mutation mode
convRates='-'	   	# Name of the conversion rate file
truncRates='-'	   	# Name of the truncation rate file
delRates='-'	   	# Name of the deletion rate file
truncLength='-'     # Name of the truncation length distribution file
Length=0			# Used for individual lower and upper boundaries
bsaffinity=1.0      # Binding site affinity 0.0 to 1.0

isUsingFiltered=false # Toggle for prefiltered annotation
pathScripts='./'    # Path to other scripts


# GETTING MODE OR PRINT HELP
mode=${args[0]}
if [ $mode == '-h' ] || [ $mode == '--help' ]; then
	set -- " " "-h"
elif ! [ $mode == 'simulateReads' ] && ! [ $mode == 'generateExpression' ] && ! [ $mode == 'generateIndex' ]; then
	echo "ERROR: $mode is not a valid mode. Please use generateIndex, generateExpression or simulateReads."
	exit 1
fi

#PARSE SETTINGS
while :; do
		case $2 in
			-h|-\?|--help)  
				
				
				cat <<-EOF
				
				Usage: $0 [mode] [options]
				
				MODES:
				
				generateIndex       Generates motif index (-i -o -m)
				generateExpression  Generates an exprssion profile (-i -o -g -G -mi)
				simulateReads       Simulates PAR-CLIP reads (-i -o -g -G -x -ml -mi)
					
				REQUIRED PARAMETERS:
					
				-g|--genomeDir	     Path to genome file in fasta format.(Optional as gziped file: genome.tar.gz if no fasta files are present in the directory)
				-o|--outDir          Path to the output directory which is also used as a working directory.
				-G|--gtf             Name of the annotation file.
				-i|--inputDir        Path to input directory containing all input files (annotation, transcriptome, flux parameter template, pattern and mutation rate files).
				
				-m|--motifFile       Name of the FIMO input file.
				-ml|--motifLen       Length of the motif.
				-c|--convRates	     Name of the conversion rate file.
				-t|--truncRates	     Name of the truncation rate file.
				-d|--delRates	     Name of the deletion rate file.
				
				OPTIONAL PARAMETERS:
				
				-r|--readsN          Maximum number of RNA-Seq reads to be created (default: 2,000,000,000). Default amount or higher is recommended.
				-y|--libName         Change the archive name the library file is stored in (default: LIB).
				-mi|--motifIdx       Set the name of FIMO output file. If used, it must be placed in the input directory.
				-s|--motifDef        Set motif to "unspecific". This changes the threshold for the motif search with FIMO (default: specific).
				-l|--readLength      Set the maximal read length (default: 30). Minimum is 11. When "-x T" is set, a tuple can be specified (i.e. 25,45).
				-rn|--runName        Sets the name of the output files of the PAR-Clip simulation (default: Runx).
				-e|--express         Sets the name of an expression profile. If used, it must be placed in the input directory.
				-u|--memory          Sets the amount of RAM used (default: 16).
				-x|--mutMode         Sets the diagnostic events that are introduced (default: C). Any combination of D, C or T seperated by a ",".
				-tl|--truncLen       Sets the name of an distribution file for trancated read lengths.
				-a|--bsaffin         Sets the binding site affinity (default: 1.0).
				
					EOF
				
				exit
				;;
			-e|--express)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					expProf=$3
					shift 2
					continue
				else
					printf 'ERROR: "-e/--express" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-a|--bsaffin)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
				bsaffinity=$3
					shift 2
					continue
				else
					printf 'ERROR: "-a/--bsaffin" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-tl|--truncLen)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					truncLength=$3
					shift 2
					continue
				else
					printf 'ERROR: "-tl/--truncLen" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-x|--mutMode)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					mutMode=$3
					shift 2
					continue
				else
					printf 'ERROR: "-x/--mutMode" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-u|--memory)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					useMemory=$3
					re='^[0-9]+$'
					if ! [[ $useMemory =~ $re ]]; then
						printf 'ERROR: "-u/--memory" requires an INT.\n' >&2
						exit 1
					fi
					shift 2
					continue
				else
					printf 'ERROR: "-u/--memory" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-mi|--motifIdx)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					motifIndex=$3
					shift 2
					continue
				else
					printf 'ERROR: "-mi/--motifIdx" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-g|--genomeDir)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					genomeDir=$3
					shift 2
					continue
				else
					printf 'ERROR: "-g/--genomeDir" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-s|--motifDef)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					motif=$3
					if ! [ $motif == "specific" ] && ! [ $motif == "unspecific" ]; then
						printf 'ERROR: "-s/--motifDef" requires "specific" or "unspecific".\n' >&2
						exit 1
					fi
					shift 2
					continue
				else
					printf 'ERROR: "-s/--motifDef" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-o|--outDir)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					outDir=$3
					shift 2
					continue
				else
					printf 'ERROR: "-o/--outDir" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-i|--inputDir)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					inputDir=$3
					shift 2
					continue
				else
					printf 'ERROR: "-i/--inputDir" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-y|--libName)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					libname=$3
					shift 2
					continue
				else
					printf 'ERROR: "-y/--libName" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-m|--motifFile)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
				motiffile=$3
					shift 2
					continue
				else
					printf 'ERROR: "-m/--motifFile" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-G|--gtf)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					gtf=$3
					shift 2
					continue
				else
					printf 'ERROR: "-G/--gtf" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-c|--convRates)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					convRates=$3
					shift 2
					continue
				else
					printf 'ERROR: "-c/--convRates" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-t|--truncRates)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					truncRates=$3
					shift 2
					continue
				else
					printf 'ERROR: "-t/--truncRates" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-d|--delRates)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					delRates=$3
					shift 2
					continue
				else
					printf 'ERROR: "-d/--delRates" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-rn|--runName)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					parname=$3
					shift 2
					continue
				else
					printf 'ERROR: "-rn/--runName" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-ml|--motifLength)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					motifLen=$3
					re='^[0-9]+$'
					if ! [[ $motifLen =~ $re ]]; then
						printf 'ERROR: "-ml/--motifLen" requires an INT.\n' >&2
						exit 1
					fi
					shift 2
					continue
				else
					printf 'ERROR: "-ml/--motifLength" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-r|--readsN)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					numR=$3
					re='^[0-9]+$'
					if ! [[ $numR =~ $re ]]; then
						printf 'ERROR: "-r/--readsN" requires an INT.\n' >&2
						exit 1
					fi
					shift 2
					continue
				else
					printf 'ERROR: "-r/--readsN" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-l|--readLength)       # Takes an option argument, ensuring it has been specified.
				if [ -n "$3" ]; then
					readLength=$3
					Length=$3
					#re='^[0-9]+$'
					#if ! [[ $readLength =~ $re ]]; then
					#	printf 'ERROR: "-l/--readLength" requires an INT.\n' >&2
					#	exit 1
					#elif [[ $readLength < 11 ]]; then
					#	printf 'ERROR: read length is to small (min. 11).\n' >&2
					#	exit 1
					#fi
					shift 2
					continue
				else
					printf 'ERROR: "-l/--readLength" requires a non-empty option argument.\n' >&2
					exit 1
				fi
				;;
			-v|--verbose)
				verbose=$((verbose + 1)) # Each -v argument adds 1 to verbosity.
				;;
			--)              # End of all options.
				shift
				break
				;;
			-?*)
					printf 'WARN: Unknown option (ignored): %s\n' "$2" >&2
				;;
			*)               # Default case: If no more options then break out of the loop.
				break
		esac

	shift
done

# CHECK REQUIRED PARAMETERS

if [ $mode == 'generateIndex' ]; then
	if [ -z "$outDir" ]; then
		printf 'ERROR: option "-o/--outDir PATH" not given. See --help.\n' >&2
		exit 1
	fi
	if [ -z "$inputDir" ]; then
		printf 'ERROR: option "-i/--inputDir PATH" not given. See --help.\n' >&2
		exit 1
	fi
	if [ -z "$motiffile" ]; then
		printf 'ERROR: option "-m/--motifFile FILE" not given. See --help.\n' >&2
		exit 1
	fi
#	if [ -z "$motifLen" ]; then
#		printf 'ERROR: option "-ml/--motifLength INT" not given. See --help.\n' >&2
#		exit 1
#	fi
elif [ $mode == 'generateExpression' ]; then
	if [ -z "$genomeDir" ]; then
		printf 'ERROR: option "-g/--genomeDir FILE" not given. See --help.\n' >&2
		exit 1
	fi
	if [ -z "$outDir" ]; then
		printf 'ERROR: option "-o/--outDir PATH" not given. See --help.\n' >&2
		exit 1
	fi
	if [ -z "$gtf" ]; then
		printf 'ERROR: option "-G/--gtf FILE" not given. See --help.\n' >&2
		exit 1
	fi
	if [ -z "$inputDir" ]; then
		printf 'ERROR: option "-i/--inputDir PATH" not given. See --help.\n' >&2
		exit 1
	fi
	if [ -z "$motifIndex" ]; then
		printf 'ERROR: option "-mi/--motifIdx FILE" not given. See --help.\n' >&2
		exit 1
	fi
elif [ $mode == 'simulateReads' ]; then
	
	split=$(echo $mutMode | tr "," "\n")
	for x in $split; do
		if [ $x == 'C' ]; then
			if [ $convRates == '-' ]; then
				printf 'ERROR: option "-c/--convRates" not given but necessary with '$mode'. See --help.\n' >&2
				exit 1
			fi
		elif [ $x == 'D' ]; then
			if [ $delRates == '-' ]; then
				printf 'ERROR: option "-d/--delRates" not given but necessary with '$mode'. See --help.\n' >&2
				exit 1
			fi
		elif [ $x == 'T' ]; then
			if [ $truncRates == '-' ]; then
				printf 'ERROR: option "-t/--truncRates" not given but necessary with '$mode'. See --help.\n' >&2
				exit 1
			fi
			if [[ $readLength = *,* ]]; then
				Length=$(echo $readLength | cut -f2 -d,)
			else
				# DOUBLE READ LENGTH TO ACCOUNT FOR TRANCATED BASES
				Length=$(($readLength * 2))
			fi
		fi
	done
	if [ -z "$genomeDir" ]; then
		printf 'ERROR: option "-g/--genomeDir FILE" not given. See --help.\n' >&2
		exit 1
	fi
	if [ -z "$outDir" ]; then
		printf 'ERROR: option "-o/--outDir PATH" not given. See --help.\n' >&2
		exit 1
	fi
	if [ -z "$gtf" ]; then
		printf 'ERROR: option "-G/--gtf FILE" not given. See --help.\n' >&2
		exit 1
	fi
	if [ -z "$inputDir" ]; then
		printf 'ERROR: option "-i/--inputDir PATH" not given. See --help.\n' >&2
		exit 1
	fi
	if [ -z "$motifIndex" ]; then 
		printf 'ERROR: option "-mi/--motifIdx FILE" not given. See --help.\n' >&2
		exit 1
	fi
	if [ -z "$motifLen" ]; then
		printf 'ERROR: option "-ml/--motifLength INT" not given. See --help.\n' >&2
		exit 1
	fi
fi


# DEBUGGING INPUT
# echo $genomeDir
# echo $outDir
# echo $gtf
# echo $motifLen
# echo $convRates
# echo $truncRates
# echo $delRates
# echo $inputDir
# echo $motiffile
# echo $parname
# exit 1

# PIPELINE
echo ""
echo "Starting Cseq-Simulator v1.02"

# CREATING OUTPUT DIRECTORY
if [ ! -e $outDir ]; then
	mkdir $outDir
fi

if [ $mode == 'generateIndex' ]; then
	echo "Generating motif index."
	
	printf "Parameters:\n\n" > $outDir"/log.txt"
	echo "	Input directory: "$inputDir >> $outDir"/log.txt"
	echo "	Output directory: "$outDir >> $outDir"/log.txt"
	echo "	FIMO input: "$inputDir/$motiffile >> $outDir"/log.txt"
	echo "	Motif specificity: "$motif >> $outDir"/log.txt"
	if [ -e $inputDir/transcripts.fasta ]; then
		echo "	Transcriptome used: "$inputDir/transcripts.fasta >> $outDir"/log.txt"
	elif [ -e $inputDir/transcripts.tar.gz ]; then
		echo "	Transcriptome used: "$inputDir/transcripts.tar.gz >> $outDir"/log.txt"
		cd $inputDir
		tar -zxf $inputDir/transcripts.tar.gz
		cd - &> /dev/null
	fi
	printf "	Output:$outDir/index.txt\n\n" >> $outDir"/log.txt"
	
	
	# TAKING TIMES
	time0=`date +%s`
	time=`date +"%T"`
	echo "[$time] Starting generation of motif index." >> $outDir"/log.txt"
	
	if [ ! -d $outDir"/FIMO-out" ]; then
		mkdir $outDir"/FIMO-out"
	fi
	
	# MOTIF SEARCH
	if [ ! -e $outDir"/FIMO-out/fimo.txt" ] && [ ! -e $outDir/fimo.txt ]; then
		if [ $motif == 'specific' ]; then
			#10G ram
			fimo -oc $outDir'/FIMO-out' --verbosity 1 --parse-genomic-coord --max-stored-scores 200000 --norc --motif-pseudo 0.01 $inputDir/$motiffile $inputDir'/transcripts.fasta'
		elif [ $motif == 'unspecific' ]; then
			#10G ram
			fimo -oc $outDir'/FIMO-out' --verbosity 1 --parse-genomic-coord --max-stored-scores 200000 --norc --motif-pseudo 0.01 --thresh 1e-3  $inputDir/$motiffile $inputDir'/transcripts.fasta'
		fi
		
		mv $outDir/FIMO-out/fimo.txt $outDir/index.txt
		rm -r $outDir/FIMO-out
	fi
	
	# TAKING TIME
	time1=`date +%s`
	time=`date +"%T"`
	echo "[$time] Index done." >> $outDir"/log.txt"
	t=$((time1-time0))
	echo "Time: $t s" >> $outDir"/log.txt"
	echo "Finished successfully!"
	
else
	# STORE NEW PARAMETER FILE PATH
	name=$outDir'/'$parname'.par'
	
	#COPY TEMPLATE TO OUTPUT DIRECTORY
	cp $inputDir'/par.par' $name
	
	echo "Starting $parname run." > $outDir"/log.txt"
	
	# STARTING ACTIONS
	if [ $mode == 'generateExpression' ]; then

		echo "Generating an expression profile."
		printf "PROGRESS: "
		
		printf "\nParameter:\n\n" >> $outDir"/log.txt"
		echo "	Input directory: "$inputDir >> $outDir"/log.txt"
		echo "	Output directory: "$outDir >> $outDir"/log.txt"
		echo "	Genome directory: "$genomeDir >> $outDir"/log.txt"
		echo "	GTF file: "$inputDir/$gtf >> $outDir"/log.txt"
		echo "	Motif index: "$inputDir/$motifIndex >> $outDir"/log.txt"
		printf "	Molecule number: $(($numR / 10))\n\n" >> $outDir"log.txt"
		
		# TAKING TIMES
		time0=`date +%s`
		time=`date +"%T"`
		echo "[$time] Starting expression profile." >> $outDir"/log.txt"
			
		# ADDING ARGUMENTS TO TEMPLATE
		echo "GEN_DIR "$genomeDir >> $name
		echo "REF_FILE_NAME "$inputDir/$gtf >> $name
		echo "READ_LENGTH "$readLength >> $name
		echo "READ_NUMBER "$numR >> $name
		echo "FASTA NO" >> $name
		echo "PAIRED_END "$ends >> $name
		molecules=$(($numR / 10))
		echo "NB_MOLECULES "$molecules >> $name
		
		# CREATE TEMP DIRECTORY
		if [ ! -e $outDir"/temp" ];then
			mkdir $outDir"/temp"
		fi
		
		# SET TEMP DIRECTORY
		export TMPDIR=$outDir"/temp"
		
		# ALLOCATE RAM FOR RNA-SEQ SIMULATION
		export FLUX_MEM=$useMemory"G"
		
		# DECOMPRESS GENOME IF RNA-SEQ SIMULATION ISN'T ALREADY DONE
		if [ ! -e $genomeDir"/*.fa*" ]; then
			cd $genomeDir
			if [ -e $genomeDir"/genome.tar.gz" ]; then
	 		 	tar -zxf $genomeDir"/genome.tar.gz"
			else
				echo 'ERROR: Could not find fasta files or a "genome.tar.gz"'" in $genomeDir."
				exit 1
			fi
			cd - &> /dev/null
		fi
	
		# TAKING TIMES
		time1=`date +%s`
		time=`date +"%T"`
		echo "[$time] Preparing genome done." >> $outDir"/log.txt"
		t=$(expr $time1 - $time0)
		echo "Time: $t s" >> $outDir"/log.txt"
		printf "*"
	
		# STARTING RNA-SEQ SIMULATION
		if [ ! -e $outDir/$parname.pro ]; then
			flux-simulator -p $name -x &> $outDir"/flux_log.txt"
			printf "*"
			time2=`date +%s`
			time=`date +"%T"`
			echo "[$time] Expression profile done." >> $outDir"/log.txt"
			t=$(expr $time2 - $time1)
			echo "Time: $t s" >> $outDir"/log.txt"
		else
			printf "\n ERROR: The file $outDir/$parname.pro already exists. No new expression simulation will be performed.\n" >&2
			time=`date +"%T"`
			echo "[$time] Skipped expression simulation.[$time]" >> $outDir"/log.txt"
			# exit 1
		fi
		if [ -e $outDir/temp ]; then
			rm -r $outDir/temp
		fi
		
		# TAKING TIME
		time2=`date +%s`
		printf "*"
		
		# REMOVING GENOME FILES
		if [ ! -e $genomeDir"/genome.fasta" ]; then
			rm $genomeDir/chr*.fa* 
		fi
		
		# TAKING TIMES
		time3=`date +%s`
		time=`date +"%T"`
		echo "[$time] Clean up genome files." >> $outDir"/log.txt"
		t=$((time3-time2))
		echo "Time: $t s" >> $outDir"/log.txt"
		printf "*"
		
		# FILTER EXPRESSION PROFILE
		if [ -e $outDir/$parname.pro ]; then
			mv $outDir/$parname.pro $outDir/$parname.pro.tmp
			python $pathScripts/filterExpression.py $inputDir/$motifIndex $outDir/$parname.pro.tmp > $outDir/$parname.pro
			rm $outDir/$parname.pro.tmp
		else
			echo "ERROR: Could not find $parname.pro in $outDir."
			exit 1
		fi
		
		rm $outDir/$parname.par
		
		# TAKING TIME
		time4=`date +%s`
		time=`date +"%T"`
		echo "[$time] Removing genome files done." >> $outDir"/log.txt"
		t=$((time4-time3))
		echo "Time: $t s" >> $outDir"/log.txt"
		printf "*\n"
		echo "$parname.pro is your expression profile."
		echo "Finished successfully!"
		
	elif [ $mode == 'simulateReads' ]; then
		
		printf "\nParameter:\n\n" >> $outDir"/log.txt"
		echo "	Input directory: "$inputDir >> $outDir"/log.txt"
		echo "	Output directory: "$outDir >> $outDir"/log.txt"
		echo "	Genome directory: "$genomeDir >> $outDir"/log.txt"
		echo "	GTF file: "$inputDir/$gtf >> $outDir"/log.txt"
		if [ -e $inputDir/transcripts.fasta ]; then
			echo "	Transcriptome used: "$inputDir/transcripts.fasta >> $outDir"/log.txt"
		elif [ -e $inputDir/transcripts.tar.gz ]; then
			echo "	Transcriptome used: "$inputDir/transcripts.tar.gz >> $outDir"/log.txt"
		fi
		echo "	Motif index: "$inputDir/$motifIndex >> $outDir"/log.txt"
		echo "	Motif length: "$motifLen >> $outDir"/log.txt"
		echo "	Motif specificity: "$motif >> $outDir"/log.txt"
		echo "	Binding site affinity: "$bsaffinity >> $outDir"/log.txt"
		echo "	Mutation mode: "$mutMode >> $outDir"/log.txt"
		if [ ! $convRates == '-' ]; then
			echo "	Conversion rates: "$inputDir/$convRates >> $outDir"/log.txt"
		fi
		if [ ! $delRates == '-' ]; then
			echo "	Deletion rates: "$inputDir/$delRates >> $outDir"/log.txt"
		fi
		if [ ! $truncRates == '-' ]; then
			echo "	Truncation rates: "$inputDir/$truncRates >> $outDir"/log.txt"
		fi
		echo "	Read length: "$readLength >> $outDir"/log.txt"
		echo "	Read number: "$numR >> $outDir"/log.txt"
		echo "	Molecule number: $(($numR / 10))" >> $outDir"log.txt"
		if [ -z $expProf ]; then
			echo "	Expression profile: "$expProf >> $outDir"/log.txt"
		fi
		printf "	Memory: "$useMemory"G\n\n" >> $outDir"/log.txt"
		
		echo "Simulating reads."
		time=`date +"%T"`
		time0=`date +%s`
		echo "[$time] Starting PAR-CLIP simulation." >> $outDir"/log.txt"
		printf "PROGRESS: "
		
		# ADDING ARGUMENTS TO TEMPLATE
		echo "GEN_DIR "$genomeDir >> $name
		echo "REF_FILE_NAME "$inputDir/$gtf >> $name
		echo "READ_LENGTH "$Length >> $name
		echo "READ_NUMBER "$numR >> $name
		echo "FASTA "$fasta >> $name
		echo "PAIRED_END "$ends >> $name
		molecules=$(($numR / 10))
		echo "NB_MOLECULES "$molecules >> $name
		echo "FRAG_UR_ETA NaN" >> $name
		echo "FRAG_UR_D0 NaN" >> $name
		#echo "SIZE_DISTRIBUTION N($readLength,0)" >> $name
		
		# CREATE TEMP DIRECTORY
		if [ ! -e $outDir"/temp" ];then
			mkdir $outDir"/temp"
		fi
		
		# SET TEMP DIRECTORY
		export TMPDIR=$outDir"/temp"
		
		# ALLOCATE RAM FOR RNA-SEQ SIMULATION
		export FLUX_MEM=$useMemory"G"
		
		# DECOMPRESS GENOME IF RNA-SEQ SIMULATION ISN'T ALREADY DONE
		if [ ! -e $genomeDir"/*.fa*" ]; then
			cd $genomeDir
			if [ -e $genomeDir"/genome.tar.gz" ]; then
				tar -zxf $genomeDir"/genome.tar.gz"
			else
				echo 'ERROR: Could not find fasta files or a "genome.tar.gz"'" in $genomeDir."
				exit 1
			fi
			cd - &> /dev/null
		fi
	
		# TAKING TIMES
		time1=`date +%s`
		time=`date +"%T"`
		echo "[$time] Preparing genome done." >> $outDir"/log.txt"
		t=$(expr $time1 - $time0)
		echo "Time: $t s" >> $outDir"/log.txt"
		printf "*"
		
		# FLUX SIMULATION WITH AND WITHOUT PREMADE EXPRESSION PROFILE
		if [ -z "$expProf" ]; then
			if [ ! -e $outDir/$parname.bed ] && [ ! -e $outDir/bed.tar.gz ]; then
				name2=$outDir'/'$parname'2.par'
				cp $inputDir'/par.par' $name2
				
				# ADDING ARGUMENTS TO TEMPLATE
				echo "GEN_DIR "$genomeDir >> $name2
				echo "REF_FILE_NAME "$inputDir/$gtf >> $name2
				echo "READ_LENGTH "$Length >> $name2
				echo "READ_NUMBER 2000000000" >> $name2
				echo "FASTA NO" >> $name2
				echo "PAIRED_END "$ends >> $name2
				echo "NB_MOLECULES 200000000" >> $name2
		 
				flux-simulator -p $name2 -x &> $outDir"/flux_log.txt"
				
				rm $name2
				mv $outDir'/'$parname'2.pro' $outDir'/'$parname'.pro'
				
				mv $outDir/$parname.pro $outDir/$parname.pro.tmp
				python $pathScripts/filterExpression.py $inputDir/$motifIndex $outDir/$parname.pro.tmp > $outDir/$parname.pro
				rm $outDir/$parname.pro.tmp
				
				printf "*"
				time2=`date +%s`
				time=`date +"%T"`
				echo "[$time] Expression file done." >> $outDir"/log.txt"
				t=$(expr $time2 - $time1)
				echo "Time: $t s" >> $outDir"/log.txt"
			    
				flux-simulator -p $name -l >> $outDir"/flux_log.txt" 2>&1
			    
			    time=`date +"%T"`
				echo "[$time] Library generation done." >> $outDir"/log.txt"
			    t2=$(expr $time - $time2)
			    echo "Time: $t2 s" >> $outDir"/log.txt"
			    time2=`date +%s`
			    
				mv $outDir/$parname.lib $outDir/$parname.lib.tmp
				python $pathScripts/filterLibrary.py $inputDir/$motifIndex $outDir/$parname.lib.tmp $Length $bsaffinity > $outDir/$parname.lib
				rm $outDir/$parname.lib.tmp
			    
				printf "*"
				time3=`date +%s`
				time=`date +"%T"`
				echo "[$time] Library filtered." >> $outDir"/log.txt"
				t=$(expr $time3 - $time2)
				echo "Time: $t s" >> $outDir"/log.txt"
			    
				flux-simulator -p $name -s >> $outDir"/flux_log.txt" 2>&1
			    
				printf "*"
				time4=`date +%s`
				time=`date +"%T"`
				echo "[$time] Sequencing done." >> $outDir"/log.txt"
				t=$(expr $time4 - $time3)
				echo "Time: $t s" >> $outDir"/log.txt"
			else
				printf "*"
				time4=`date +%s`
				time=`date +"%T"`
				echo "[$time] Using already simulated." >> $outDir"/log.txt"
			fi
		else
			if [ ! -e $outDir/$parname.bed ] && [ ! -e $outDir/bed.tar.gz ]; then
				cp $inputDir/$expProf $outDir/$parname.pro
				flux-simulator -p $name -l &> $outDir"/flux_log.txt"
				
				mv $outDir/$parname.lib $outDir/$parname.lib.tmp
				python $pathScripts/filterLibrary.py $inputDir/$motifIndex $outDir/$parname.lib.tmp $Length $bsaffinity > $outDir/$parname.lib
				rm $outDir/$parname.lib.tmp
				
				printf "*"
				time2=`date +%s`
				time=`date +"%T"`
				echo "[$time] Library filtered." >> $outDir"/log.txt"
				t=$(expr $time2 - $time1)
				echo "Time: $t s" >> $outDir"/log.txt"
				
				flux-simulator -p $name -s >> $outDir"/flux_log.txt" 2>&1
				
				printf "*"
				time3=`date +%s`
				time4=`date +%s`
				time=`date +"%T"`
				echo "[$time] Sequencing done." >> $outDir"/log.txt"
				t=$(expr $time3 - $time2)
				echo "Time: $t s" >> $outDir"/log.txt"
			else
				printf "*"
				time4=`date +%s`
				time=`date +"%T"`
				echo "[$time] Using already simulated data." >> $outDir"/flux_log.txt" 2>&1
			fi
		fi
		if [ -e $outDir/temp ]; then
			rm -r $outDir/temp
		fi
		
		# MOVE THE LIB AND PARAMETER FILE, COMPRESS IT, AND DELETE UNCOMPRESSED
		if [ ! -e $outDir"/"$libname".tar.gz" ]; then
			mv $name $outDir'/parameter.par'
			tar -zcf $outDir/$libname'.tar.gz' -C $outDir parameter.par $parname.lib
			rm $outDir/parameter.par
			rm $outDir/$parname.lib
		fi
		
		# REMOVING GENOME FILES
		if [ ! -e $genomeDir"/genome.fasta" ]; then
			rm $genomeDir/chr*.fa*
		fi
		
		# TAKING TIME
		time5=`date +%s`
		time=`date +"%T"`
		echo "[$time] Cleaning up files." >> $outDir"/log.txt"
		t=$((time5-time4))
		echo "Time: $t s" >> $outDir"/log.txt"
		printf "*"
		
		# EDITING READS
		if [ ! -e $outDir/reads.tmp ]; then
			python editReads.py $outDir/$parname.bed $outDir/$parname.fasta > $outDir/reads.tmp
		fi
		
		
		# TAKING TIME
		time6=`date +%s`
		time=`date +"%T"`
		echo "[$time] Preparation of reads done." >> $outDir"/log.txt"
		t=$((time6-time5))
		echo "Time: $t s" >> $outDir"/log.txt"
		printf "*"
		
		# INDEXING
		if [ ! -e $outDir/index.tmp ]; then
			python indexing.py $inputDir $inputDir/$motifIndex $motifLen > $outDir/index.tmp
		fi
		
		# TAKING TIME
		time7=`date +%s`
		time=`date +"%T"`
		echo "[$time] Indexing motifs done." >> $outDir"/log.txt"
		t=$((time7-time6))
		echo "Time: $t s" >> $outDir"/log.txt"
		printf "*"
		
		if [ ! -e $outDir/$parname.bed ]; then
			if [ -e $outDir/bed.tar.gz ]; then
				cd $outDir
				tar -zxf $outDir/bed.tar.gz
				cd - &> /dev/null
			else
				echo "ERROR: Counld not find $parname.bed or bed.tar.gz in $outDir ."
				exit 1
			fi
		fi
		if [ ! -e $outDir/$parname.fasta ]; then
			if [ -e $outDir/uneditedSequences.tar.gz ]; then
				cd $outDir
				tar -zxf $outDir/uneditedSequences.tar.gz
				cd - &> /dev/null
			else
				echo "ERROR: Counld not find $parname.fasta or uneditedSequences.tar.gz in $outDir ."
				exit 1
			fi
		fi
	
		# MUTATE READS
		if [ ! -e $outDir"/mutated_Reads"$parname".fasta" ];then
			python $pathScripts/mutReadsUniversal.py $outDir/reads.tmp $inputDir"/"$convRates $inputDir"/"$delRates $inputDir"/"$truncRates $outDir/index.tmp $outDir"/unmutated_Reads"$parname".fasta" $outDir"/read_Log.txt" $motifLen $inputDir/$gtf $mutMode $readLength $inputDir/$truncLength > $outDir"/mutated_Reads"$parname".fasta"
			tar -zcf $outDir/mutated_Reads$parname.tar.gz -C $outDir mutated_Reads$parname.fasta
			tar -zcf $outDir/unmutated_Reads$parname.tar.gz -C $outDir unmutated_Reads$parname.fasta
			mv $outDir/reads.tmp $outDir/Reads.fasta
			tar -zcf $outDir/Reads$parname.tar.gz -C $outDir Reads.fasta
			rm $outDir"/unmutated_Reads"$parname".fasta"
			rm $outDir"/mutated_Reads"$parname".fasta"
			mv $outDir/index.tmp $outDir/indexing.txt
			#rm $outDir/Reads.fasta
		fi
		
		if [ -e $outDir/$parname.bed ]; then
			if [ ! -e $outDir/bed.tar.gz ]; then
				cd $outDir
				tar -zcf $outDir/bed.tar.gz -C $outDir $parname.bed
				cd - &> /dev/null
				rm $outDir/$parname.bed
			else
				rm $outDir/$parname.bed
			fi
		fi
		if [ -e $outDir/$parname.fasta ]; then
			if [ ! -e $outDir/uneditedSequences.tar.gz ]; then
				cd $outDir
				tar -zcf $outDir/uneditedSequences.tar.gz -C $outDir $parname.fasta
				cd - &> /dev/null
				rm $outDir/$parname.fasta
			else
				rm $outDir/$parname.fasta
			fi
		fi
		
		# TAKING TIME
		time8=`date +%s`
		echo "[$time] Reads are done." >> $outDir"/log.txt"
		t=$((time8-time7))
		echo "Time: $t s" >> $outDir"/log.txt"
		printf "*\n"
		echo "Finished successfully!"
		
	fi
fi

exit